#pragma once

#include <algorithm>
#include <stdexcept>

namespace provided {
    constexpr intmax_t power_of_10(const intmax_t exponent) noexcept {
        auto result = 10;
        for (intmax_t i = 1; i < exponent; ++i) {
            result *= 10;
        }
        return result;
    }

    constexpr intmax_t power(intmax_t const radix, intmax_t const exponent) noexcept {
        auto result = radix;
        for (intmax_t i = 1; i < exponent; ++i) {
            result *= radix;
        }
        return result;
    }

    template <char digit, char... digits>
    struct literal {
        static constexpr auto value = literal<digit>::value * power_of_10(sizeof...(digits)) + literal<digits...>::value;
    };

    template <char digit>
    struct literal<digit> {
        static const constexpr auto value = digit - '0';
    };

    class static_exception : public std::exception {
    public:
        static_exception(const char * const msg = "")
            :m_msg(msg) {}
        const char* what() const noexcept override {
            return m_msg;
        }
    private:
        const char* m_msg;
    };
}

#if defined(THROWING_STATIC_ASSERT)
#define TESTABLE_STATIC_ASSERT(cond, msg)                \
    do {                                                 \
        if (!(cond)) {                                   \
                throw provided::static_exception(msg);   \
        }                                                \
    } while (0)

#else
#define TESTABLE_STATIC_ASSERT(cond, msg) static_assert(cond, msg)
#endif




//////////////////////////
// Note: This must be below the definition of bounded_int<a, b>!
////
namespace provided {
    template <long Value>
    using constant = bounded_int<Value, Value>;

    namespace literals {
        template<char... digits>
        auto operator ""_bi() noexcept {
            return constant<literal<digits...>::value>(literal<digits...>::value);
        }
    }
}
